import sems
import os

from sems.server import make_app
from sems.repository.monitor import Monitor

from tornado.testing import AsyncHTTPTestCase


class TestDashboard(AsyncHTTPTestCase):

    def tearDown(self):
        if os.path.isfile(sems.repository.base.TINY_DB_PATH):
            os.remove(sems.repository.base.TINY_DB_PATH)

    def get_app(self):
        return make_app()

    def test_dashboard(self):
        label = "server.test"
        monitor = Monitor(label=label, url='http://server.com')
        monitor.save()

        response = self.fetch('/')

        self.assertEqual(response.code, 200)
        self.assertIn(label, response.body)
        self.assertIn('TextMonitor', response.body)
        self.assertIn('Default', response.body)

    def test_dashboard_with_group(self):
        label = "server.test"
        monitor = Monitor(label=label, url='http://server.com', group='test')
        monitor.save()

        response = self.fetch('/')

        self.assertEqual(response.code, 200)
        self.assertIn(label, response.body)
        self.assertIn('TextMonitor', response.body)
        self.assertIn('test', response.body)
        self.assertNotIn('Default', response.body)




"""
Simple Environment Monitor System settings

For more information about available settings look at
https://github.com/diegorubin/simple-environment-monitor-system
"""
from os.path import abspath, curdir, join
from os import environ

TINY_DB_PATH = environ.get("SEMS_DATABASE", join(abspath(curdir), "db.json"))
SERVER_PORT = int(environ.get("SEMS_SERVER_PORT", "8888"))
LOG_PATH = environ.get("SEMS_LOG_PATH",join(abspath(curdir), "log"))
LOG_LEVEL = environ.get("SEMS_LOG_LEVEL", "INFO")
DEBUG = environ.get("SEMS_DEBUG", "False") == "True"





import sems, os

from sems.server import make_app
from sems.repository.monitor import Monitor

from tornado.escape import json_encode
from tornado.testing import AsyncHTTPTestCase

SERVICE_URL = "http://localhost:8888/healthcheck"


class TestMonitors(AsyncHTTPTestCase):

    def tearDown(self):
        if os.path.isfile(sems.repository.base.TINY_DB_PATH):
            os.remove(sems.repository.base.TINY_DB_PATH)

    def get_app(self):
        return make_app()

    def test_custom_fields(self):
        response = self.fetch('/api/monitors/TextMonitor/fields')
        self.assertEqual(response.code, 200)
        self.assertIn('expected', response.body)

    def test_create(self):
        body = json_encode({
            'type': 'TextMonitor', 'label': 'server',
            'url': SERVICE_URL,
            'data': {'expected': 'LIVE'}
        })
        response = self.fetch('/api/monitors', method='POST', body=body)
        self.assertEqual(response.code, 200)
        self.assertEqual(len(Monitor().all()), 1)

    def test_list(self):
        label = 'test'

        monitor = Monitor(label=label)
        monitor.save()

        response = self.fetch('/api/monitors')
        self.assertEqual(response.code, 200)
        self.assertIn(label, response.body)

    def test_check(self):
        label = 'test'

        monitor = Monitor(label=label,
                          url=SERVICE_URL,
                          monitor_type='TextMonitor',
                          data={'expected': 'LIVE'})
        monitor.save()

        response = self.fetch('/api/monitors/%s/check' % label)
        self.assertEqual(response.code, 200)
        self.assertIn('false', response.body)

    def test_positions_save(self):
        monitor_position1 = Monitor(label='position1',
                                   url=SERVICE_URL,
                                   monitor_type='TextMonitor',
                                   data={'expected': 'LIVE'})
        monitor_position1.save()

        monitor_position0 = Monitor(label='position0',
                                   url=SERVICE_URL,
                                   monitor_type='TextMonitor',
                                   data={'expected': 'LIVE'})
        monitor_position0.save()

        body = json_encode({
            'monitors': ['position0', 'position1']
        })

        self.fetch('/api/monitors/positions', method='POST', body=body)
        monitor = Monitor()
        monitor.load('position1')
        self.assertEqual(monitor.get_position(), 1)

        monitor = Monitor()
        monitor.load('position0')
        self.assertEqual(monitor.get_position(), 0)





import tornado
import os

from sems import logger
from sems.logger import general

from sems.server.api.monitors import *
from sems.server.dashboard import Dashboard
from sems.server.static import Static

from sems.settings import SERVER_PORT, DEBUG


def make_app():
    app = tornado.web.Application(
        [
            (r"/", Dashboard),
            (r"/static/([\w_\.-]+)", Static),
            (r"/api/monitors", MonitorsHandler),
            (r"/api/monitors/positions", MonitorsPositionsHandler),
            (r"/api/monitors/(.+)/check", MonitorsCheckHandler),
            (r"/api/monitors/(\w+)/fields", MonitorsFieldsHandler),
            (r"/api/monitors/(.+)", MonitorsDestroyHandler),
        ],
        template_path=os.path.join(os.path.dirname(__file__), "templates"),
        static_path=os.path.join(os.path.dirname(__file__), "static"),
        debug=DEBUG
    )
    return app


def start():

    logger.initialize_logging()

    app = make_app()
    app.listen(SERVER_PORT)

    general.info("starting server")

    tornado.ioloop.IOLoop.current().start()




import sys

try:
    from urllib import error
except ImportError:
    import urllib2 as error

from sems.monitors.text_monitor import TextMonitor
from sems.monitors.http_status_monitor import HTTPStatusMonitor
from sems.monitors.socket_port_monitor import SocketPortMonitor

CUSTOM_FIELDS = {
    r'TextMonitor': {
        'expected': {'type': 'text'}
    },
    r'HTTPStatusMonitor': {
        'code': {'type': 'text'}
    },
    r'SocketPortMonitor': {
        'port': {'type': 'text'}
    }
}


def check_alive(monitor_type, url, **data):
    klass = getattr(sys.modules[__name__], monitor_type)
    monitor = klass(url, **data)

    try:
        return monitor.alive()
    except error.URLError:
        return False


def get_custom_fields(monitor_type):
    return CUSTOM_FIELDS[monitor_type]




import unittest
import httpretty

from sems.monitors.socket_port_monitor import SocketPortMonitor
from sems.monitors import check_alive


SERVICE_HOST = "localhost"
SERVICE_PORT = "8888"
SERVICE_URL = "http://%s:%s/" % (SERVICE_HOST, SERVICE_PORT)


class TestSocketPortMonitor(unittest.TestCase):

    def test_alive(self):
        httpretty.enable()
        httpretty.register_uri(httpretty.GET, SERVICE_URL, body="LIVE")

        monitor = SocketPortMonitor(SERVICE_HOST, port=SERVICE_PORT);
        self.assertTrue(monitor.alive())

        httpretty.disable()
        httpretty.reset()

    def test_out(self):
        monitor = SocketPortMonitor('hostnotexists', port=SERVICE_PORT);
        self.assertFalse(monitor.alive())

    def test_check_alive(self):
        httpretty.enable()
        httpretty.register_uri(httpretty.GET, SERVICE_URL, body="LIVE")

        self.assertTrue(check_alive('SocketPortMonitor', SERVICE_HOST, port='8888'))

        httpretty.disable()
        httpretty.reset()




# -*- coding: utf-8 -*-

import tornado.web

from sems.repository.monitor import Monitor


class Dashboard(tornado.web.RequestHandler):

    def get(self):

        monitor_types = [
            ('Select...', ''),
            ('Check HTTP Response Content', 'TextMonitor'),
            ('Check Status HTTP', 'HTTPStatusMonitor'),
            ('Check Socket Port', 'SocketPortMonitor')
        ]

        monitors = {}
        for monitor in Monitor().ordered_by_position():
            if hasattr(monitor, "group"):
                group = monitor.group
            else:
                group = ""

            if group not in monitors:
                monitors[group] = []
            monitors[group].append(monitor)

        groups = set([monitor for monitor in monitors])

        self.render("dashboard.html", monitor_types=monitor_types, groups_monitors=monitors, groups=groups)





import os

from tornado.log import LogFormatter
from tornado.log import logging

from sems import settings

general = logging.getLogger('tornado.general')
application = logging.getLogger('tornado.application')
access = logging.getLogger('tornado.access')


def initialize_logging():

    for stream_name in ['access', 'general', 'application']:
        stream = logging.getLogger("tornado.%s" % stream_name)

        stream.setLevel(getattr(logging, settings.LOG_LEVEL))
        if not os.path.exists(settings.LOG_PATH):
            os.makedirs(settings.LOG_PATH)

        channel = logging.handlers.RotatingFileHandler(
            filename=os.path.join(settings.LOG_PATH, "sems.%s.log" % stream_name),
        )

        channel.setFormatter(LogFormatter(color=True))
        stream.addHandler(channel)




try:
    from urllib import error
except ImportError:
    import urllib2 as error

from sems.monitors.base import Base


class TextMonitor(Base):

    def alive(self):

        try:
            content = self.get_content().decode('utf-8')
            return self.data['expected'] in content
        except error.HTTPError:
            return False
        except error.URLError:
            return False



from sems.server.api.base import Base

from tornado.escape import json_decode

from sems.repository.monitor import Monitor
from sems.monitors import check_alive, get_custom_fields
from sems.logger import application

class MonitorsHandler(Base):

    def get(self):
        monitors = []

        for monitor in Monitor().all():
            monitors.append(monitor.get_attributes())

        self.success({'monitors': monitors})

    def post(self):

        application.info("creating monitor with attributes %s" % self.request.body)
        json_request = json_decode(self.request.body)
        monitor = Monitor(**json_request)
        monitor.save()

        self.success({"monitor": monitor.get_attributes()})


class MonitorsPositionsHandler(Base):

    def post(self):
        json_request = json_decode(self.request.body)

        for index, label in enumerate(json_request['monitors']):
            monitor = Monitor()
            monitor.load(label)
            monitor.position = index
            monitor.save()

        self.success(json_request)


class MonitorsDestroyHandler(Base):

    def delete(self, label):
        monitor = Monitor()
        monitor.load(label)

        application.info("removing monitor %s" % label)

        self.success({"monitor": monitor.get_attributes(), "destroy": monitor.destroy()})


class MonitorsCheckHandler(Base):

    def get(self, label):
        monitor = Monitor()
        monitor.load(label)

        alive = check_alive(monitor.monitor_type, monitor.url, **monitor.data)
        self.success({"monitor": monitor.get_attributes(), "alive": alive})


class MonitorsFieldsHandler(Base):

    def get(self, monitor_type):

        self.success({"fields": get_custom_fields(monitor_type)})




import telnetlib

from sems.monitors.base import Base


class SocketPortMonitor(Base):

    def alive(self):

        try:
            connection = telnetlib.Telnet(self.entrypoint, int(self.data['port']), 1)
            connection.close()
            return True
        except:
            return False



import unittest
import shutil
import os

from sems import settings
from sems import logger


class TestLogger(unittest.TestCase):

    def setUp(self):
        settings.LOG_PATH = "tests/log"

    def tearDown(self):
        shutil.rmtree("tests/log")

    def test_files_created(self):
        logger.initialize_logging()

        self.assertTrue(os.path.isfile("tests/log/sems.access.log"))
        self.assertTrue(os.path.isfile("tests/log/sems.application.log"))
        self.assertTrue(os.path.isfile("tests/log/sems.general.log"))




try:
    from urllib import error
except ImportError:
    import urllib2 as error

from sems.monitors.base import Base


class HTTPStatusMonitor(Base):

    def alive(self):

        try:
            response = self.get_url_response()
            return response.getcode() == int(self.data['code'])
        except error.HTTPError as e:
            if hasattr(e, 'code'):
                return e.code == int(self.data['code'])
            else:
                return False
        except error.URLError:
            return False



from tinydb import TinyDB, Query
from sems.settings import TINY_DB_PATH


class Base(object):

    def __init__(self, **attributes):
        self.__dict__ = attributes
        self.db = TinyDB(TINY_DB_PATH)
        self.table = self.db.table(self.__class__.__name__)

    def all(self):
        monitors = []
        for elem in self.table.all():
            monitors.append(self.__class__(**elem))
        return monitors

    def load(self, label):
        query = Query()
        self.__dict__.update(self.table.search(query.label == label)[0])

    def save(self):
        attributes = self.get_attributes()
        if self.__new_record__():
            self.table.insert(attributes)
        else:
            query = Query()
            self.table.update(attributes, query.label == self.label)
        return True

    def destroy(self):
        query = Query()
        self.table.remove(eids=[self.table.search(query.label == self.label)[0].eid])

    def get_attributes(self):
        attributes = self.__dict__.copy()
        attributes.pop('db')
        attributes.pop('table')

        return attributes

    def __new_record__(self):
        query = Query()
        return len(self.table.search(query.label == self.label)) == 0



from sems.repository.base import Base


class Monitor(Base):

    def get_position(self):
        if hasattr(self, "position"):
            return self.position
        else:
            return 0

    def ordered_by_position(self):
        return sorted(self.all(), key=lambda m: m.get_position())



import unittest
import sems
import os

from sems.repository.event import Event


class TestEvent(unittest.TestCase):

    def tearDown(self):
        os.remove(sems.repository.base.TINY_DB_PATH)

    def test_insert(self):
        event = Event(label="TEST")
        self.assertTrue(event.save())

    def test_all(self):
        event = Event(label="TEST")
        event.save()
        self.assertEqual(len(event.all()), 1)

    def test_find(self):
        Event(label="TEST").save()

        event = Event()
        event.load("TEST")
        self.assertEqual("TEST", event.label)

    def test_update(self):
        Event(label="TEST", name="name1").save()

        event = Event()
        event.load("TEST")
        event.name = "name2"
        event.save()

        event = Event()
        event.load("TEST")

        self.assertEqual(len(event.all()), 1)
        self.assertEqual(event.name, "name2")

    def test_destroy(self):
        Event(label="TEST", name="name1").save()

        event = Event()
        event.load("TEST")
        event.destroy()

        self.assertEqual(len(Event().all()), 0)



import unittest

from sems.repository.monitor import Monitor


class TestMonitor(unittest.TestCase):

    def test_get_default_position(self):
        monitor = Monitor()
        self.assertEqual(monitor.get_position(), 0)

    def test_get_custom_position(self):
        monitor = Monitor(position=2)
        self.assertEqual(monitor.get_position(), 2)



from abc import abstractmethod

try:
    from urllib import request
except ImportError:
    import urllib2 as request


class Base(object):

    def __init__(self, entrypoint, **data):
        self.entrypoint = entrypoint
        self.data = data

    @abstractmethod
    def alive(self):
        pass

    def get_url_response(self):
        return request.urlopen(self.entrypoint)

    def get_content(self):
        return self.get_url_response().read()




import unittest
import httpretty

from sems.monitors.http_status_monitor import HTTPStatusMonitor
from sems.monitors import check_alive


SERVICE_URL = "http://localhost:8888/healthcheck"


class TestHTTPStatusMonitor(unittest.TestCase):

    def setUp(self):
        httpretty.enable()

    def tearDown(self):
        httpretty.disable()
        httpretty.reset()

    def test_alive(self):
        httpretty.register_uri(httpretty.GET, SERVICE_URL, status=200)
        monitor = HTTPStatusMonitor(SERVICE_URL, code='200');
        self.assertTrue(monitor.alive())

    def test_out(self):
        monitor = HTTPStatusMonitor(SERVICE_URL+"/false", status='200');
        self.assertFalse(monitor.alive())

    def test_404_status(self):
        httpretty.register_uri(httpretty.GET, SERVICE_URL, status=404)
        monitor = HTTPStatusMonitor(SERVICE_URL, code='404');
        self.assertTrue(monitor.alive())

    def test_422_status(self):
        httpretty.register_uri(httpretty.GET, SERVICE_URL, status=422)
        monitor = HTTPStatusMonitor(SERVICE_URL, code='422');
        self.assertTrue(monitor.alive())

    def test_expected_500_status(self):
        httpretty.register_uri(httpretty.GET, SERVICE_URL, status=500)
        monitor = HTTPStatusMonitor(SERVICE_URL, code='500');
        self.assertTrue(monitor.alive())

    def test_not_expected_500_status(self):
        httpretty.register_uri(httpretty.GET, SERVICE_URL, status=500)
        monitor = HTTPStatusMonitor(SERVICE_URL, code='200');
        self.assertFalse(monitor.alive())

    def test_check_alive(self):
        httpretty.register_uri(httpretty.GET, SERVICE_URL, status=200)
        self.assertTrue(check_alive('HTTPStatusMonitor', SERVICE_URL, code='200'))




import unittest
import httpretty

from sems.monitors.text_monitor import TextMonitor
from sems.monitors import check_alive


SERVICE_URL = "http://localhost:8888/healthcheck"


class TestTextMonitor(unittest.TestCase):

    def setUp(self):
        httpretty.enable()
        httpretty.register_uri(httpretty.GET, SERVICE_URL, body="LIVE")

    def tearDown(self):
        httpretty.disable()
        httpretty.reset()

    def test_alive(self):
        monitor = TextMonitor(SERVICE_URL, expected='LIVE');
        self.assertTrue(monitor.alive())

    def test_out(self):
        monitor = TextMonitor(SERVICE_URL+"/false", expected='LIVE');
        self.assertFalse(monitor.alive())

    def test_check_alive(self):
        self.assertTrue(check_alive('TextMonitor', SERVICE_URL, expected='LIVE'))





from setuptools import setup, find_packages

setup(
    name="sems",
    version="0.0.12.1",
    url="http://github.com/diegorubin/simple-environment-monitor-system",
    author="Diego Rubin",
    author_email="rubin.diego@gmail.com",
    license="GPL2",
    scripts=['bin/sems-start'],
    include_package_data=True,
    description="A Simple Environment Monitor System",
    install_requires=[
        "tornado",
        "tornado_json",
        "tinydb"
    ],
    classifiers=["Development Status :: 3 - Alpha"],
    packages=find_packages()
)

